;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; Loading this file installs CG/JS.
;; Please see readme.txt for more information
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(in-package :user)

(eval-when (compile eval load)
  (require :shell)
  (use-package :excl.shell))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; validate files

(let ((platform-file
       (directory
	(merge-pathnames
	 "PLATFORM-*"
	 (path-pathname *load-pathname*)))))
  (or platform-file
      (error "Could not find the PLATFORM file."))
  
  (when (> (length platform-file) 1)
    (error "There is more than one PLATFORM file: ~a." platform-file))
  (setq platform-file (car platform-file))
  
  (or (=~ "^PLATFORM-(.*)" (file-namestring platform-file))
      (error "Cannot parse the PLATFORM file."))
  
  (let ((expected-platform $1)
	(current-platform
	 (format nil "~a~a" (symbol-name sys::.os.) #+smp "smp" #-smp "")))
    (or (string-equal expected-platform current-platform)
	(error
	 "The current platform (~a) does not match the downloaded version (~a)."
	 current-platform expected-platform)))
  ;; We are on the correct platform, fall through...
  )

;; HTML files needed for the operation of CG/JS
;; (this variable's value is generated by build_cgjs.sh)
(defvar *html-files* '("cgjs.html" "ssl-not-found.html" "browser-not-supported.html" ))

(defvar *extra-files*)
(setq *extra-files* nil)

(dolist (file *html-files*)
  (setq file (merge-pathnames file *load-pathname*))
  (or (probe-file file)
      (error "~a does not exist." file))
  (push file *extra-files*))
;; All *html-files* exist...

;; These two are directories but the pathname is specified as
;; a file, so merging works below.
(defvar *cg2js-directory* (merge-pathnames "code.cg2js" *load-pathname*))
(defvar *idejs-directory* (merge-pathnames "code.idejs" *load-pathname*))

(dolist (d (list *cg2js-directory* *idejs-directory*))
  (or (probe-file d)
      (error "~a does not exist." d)))

;; MD5 checksums of the incoming files
;; (this variable's value is generated by build_cgjs.sh)
(defvar *checksums* 
'(
(#x47ac260861e0c3860ce4ce6f7518c6db . "./code.cg2js/cg-bitmap-pane-clipboard.fasl")
(#xea70ddff34ebe056457dd70fda0caf11 . "./code.cg2js/cg-drag-and-drop.fasl")
(#x975efb168215940b6be2b517c84ed449 . "./code.cg2js/cg-toolbar.fasl")
(#x567b77a4242777bfbc4e3b9408d3d1ca . "./code.cg2js/cg-chart-or-plot.fasl")
(#x15282e1dc2352d79f778ac43d31611c4 . "./code.cg2js/cg-rich-edit-pane-printing.fasl")
(#x3ce1f0388228a0c39fb5870a4fc0a07c . "./code.cg2js/cg-class-grid.fasl")
(#xac3f318160436a8747bb8567e234542e . "./code.cg2js/cg-tray.fasl")
(#x05617b8089060ed8b53ed35bd2dcc526 . "./code.cg2js/cg-gesture-emulation.fasl")
(#x81224633664c817ace472589f7f4c41a . "./code.cg2js/cg-class-support.fasl")
(#x7c26c53d079d0239265ad16d764723be . "./code.cg2js/cg-multi-line-lisp-text.fasl")
(#xb2b6ae5ae803e397d48bf101d4a7d918 . "./code.cg2js/cg-dropping-outline.fasl")
(#xdd26be5fb7c1c637f1b22025e8d20dad . "./code.cg2js/cg-rich-edit-pane.fasl")
(#x97401df4b2900457b7e63856f10f5d13 . "./code.cg2js/cg-choice-list.fasl")
(#x2ea84ab85d8b16299aa89d22b3b9c4d0 . "./code.cg2js/cg-grid-widget-drag-and-drop.fasl")
(#xf7fae26a4fc45a490bf1fd8b023b62a5 . "./code.cg2js/cg-picture-widget-palette.fasl")
(#xa368dc39b22a5d105b35598948bb00a7 . "./code.cg2js/cg-timer.fasl")
(#x4df38ecc9a06e4284fe8858a2ef8e1b4 . "./code.cg2js/cg-directory-dialog.fasl")
(#x569f93603f8f5b5f398bcbec6fe59cfe . "./code.cg2js/cg-multi-picture-button-tooltip.fasl")
(#xa5f0b79f5ec1079d98370c3fe3fd2e3a . "./code.cg2js/cg-outline-edit-in-place.fasl")
(#xc4c54d2ef2ed235118b684de973732ba . "./code.cg2js/cg-pixmap-rotate.fasl")
(#xf6ce6ed29d40468aaa1642e7dabf97a8 . "./code.cg2js/cg-html-widget.fasl")
(#x50475face49e408dd1b47fc4506056f3 . "./code.cg2js/cg-selected-object.fasl")
(#x51073b8e804de89d92c66e6973a1d9a5 . "./code.cg2js/cg-lamp.fasl")
(#x06f6da0f2e365b6251089654f4ee80fb . "./code.cg2js/cg-scroll-bar.fasl")
(#xfbee2630a03318e6e5550207b76c17cd . "./code.cg2js/cg-cursor-pixmap.fasl")
(#x805f4623c89c5066eef87c3e33cf1649 . "./code.cg2js/cg-message-dialog.fasl")
(#x4c9a085eab1b280bc661cc7d885427ec . "./code.cg2js/cg-dde-utils.fasl")
(#x2808f1325e207c6ea89da9de5dc058a2 . "./code.cg2js/cg-group-box.fasl")
(#x2a52f839f6aac070981ca0d5f2806279 . "./code.cg2js/cg-find-string-dialog.fasl")
(#x8ea0e2949d0c5cc55a620d6a6464b6e1 . "./code.cg2js/cg-split-bar.fasl")
(#x5c135f548b068fb98714975f46f8b9c7 . "./code.cg2js/cg-radio-button.fasl")
(#x1f6e5ca12feeb672bcb9272c180b9ace . "./code.cg2js/cg-base.fasl")
(#x34cdf3ae481c587200349a0c33d8c049 . "./code.cg2js/cg-paren-matching.fasl")
(#xa64462effa376c7e736c77c88e3650ab . "./code.cg2js/cg-graphics-context.fasl")
(#xf2f5cdc3960d6575702daed1f7fd7cad . "./code.cg2js/cg-keyboard-shortcuts.fasl")
(#xd9679667aaf29e7303294d1b5d8cf296 . "./code.cg2js/cg-scroll-bar-mixin.fasl")
(#x3df630a4c7ff0c699f419aa1681d7317 . "./code.cg2js/cg-plot-widget.fasl")
(#xf8b81194b4af81e92bc63094709a7434 . "./code.cg2js/cg-dev.fasl")
(#xcca8eb4041e5d9a4c3e17b9f3140802b . "./code.cg2js/cg-text-edit-pane.fasl")
(#x02233bc76e751a099e8102b283aaf06e . "./code.cg2js/cg-tab-control.fasl")
(#x5b58c17518382b673997eaefda0d6d45 . "./code.cg2js/cg-fill-texture.fasl")
(#xd6c3bda0d6bbe3417fd9be897dbebc3a . "./code.cg2js/cg-menu-tooltip.fasl")
(#x306a45fd737951386be1e4067bf85340 . "./code.cg2js/cg-trackbar.fasl")
(#x908ae353c7f0fbd5cf8a612f6e7425cb . "./code.cg2js/cg-nodes.fasl")
(#x284666388bfd55e80a9d14a4b8e0396b . "./code.cg2js/cg-common-control.fasl")
(#xc0dfe2cbeddf0479fb1c24bffacda2bc . "./code.cg2js/cg-list-view.fasl")
(#x4a8049027f258640d7192f253f84fac0 . "./code.cg2js/cg-rich-edit.fasl")
(#x3fb58772bb2a0939fff4285ea28245d9 . "./code.cg2js/cg-project-window.fasl")
(#xafaa4c776ae281628943d409e80c96ed . "./code.cg2js/cg-item-list.fasl")
(#xf950ef885b39bd5dbc280ea7fad2f33b . "./code.cg2js/cg-multi-picture-button-drag-and-drop.fasl")
(#x72b595b9edbc107f429067eaa407bd25 . "./code.cg2js/cg-get-position.fasl")
(#xb62c9509c4c0a98fa32d184c9765873b . "./code.cg2js/cg-bitmap-pane.fasl")
(#x9cd3b36ef282f7a2216ca40136acf748 . "./code.cg2js/cg-drawable.fasl")
(#x51e12b2a14a8c8ba0bf71caa25b0c949 . "./code.cg2js/cg-lettered-menu.fasl")
(#xa800a001c096ab5d0e42eea2073c4999 . "./code.cg2js/cg-multi-line-editable-text.fasl")
(#x30b4cc7beb118f375ef6eab3f04984d4 . "./code.cg2js/cg-html-dialog.fasl")
(#x6ea83c94716ca5fe5b4f75e563f87a20 . "./code.cg2js/cg-object-editor-layout.fasl")
(#x4a6720471069bb0bd0c760e6f228e741 . "./code.cg2js/cg-utility-dialog.fasl")
(#x09be47d42c19014cba9f8b613f3a6e06 . "./code.cg2js/cg-pixmap-widget.fasl")
(#x5a18ff354e42e3ebb619f663937c35c9 . "./code.cg2js/cg-property.fasl")
(#x0629f9fa1ab9d42195019337423b8b08 . "./code.cg2js/cg-menu.fasl")
(#x96b78605ec04cf9ef51475b46af9a04e . "./code.cg2js/cg-clipboard.fasl")
(#xf2e3026893b1fae2098830b427249ddf . "./code.cg2js/cg-toggling-widget.fasl")
(#xf098eb106082774b7925d3a41c4b002c . "./code.cg2js/cg-rich-edit-pane-clipboard.fasl")
(#x8ba7cdfebe03762c2918940e85ea7e76 . "./code.cg2js/cg-template-string.fasl")
(#xfa1341df78d6d648643b43b750d991a4 . "./code.cg2js/cg-drag-and-drop-image.fasl")
(#x0e425611114b36505045c07a15781818 . "./code.cg2js/cg-os-window.fasl")
(#xf8e3d928f59cae350035354fa554de27 . "./code.cg2js/cg-grid-widget.fasl")
(#xc025b3a09cc6da87676411ea23ab46bb . "./code.cg2js/cg-picture-widget.fasl")
(#x29d89ef6f4f74b76a87a7af37e32511c . "./code.cg2js/cg-text-edit-pane-mark.fasl")
(#x234ff6b70afc3a85a09ab864f8f02648 . "./code.cg2js/cg-static-text.fasl")
(#x123a0f0fbbf1a53aa91c86063b10c061 . "./code.cg2js/cg-clipboard-stack.fasl")
(#x003813107a4ad71b0fc032825be38f98 . "./code.cg2js/cg-text-widget.fasl")
(#x0784094ca2aaf30c0faf313aa60295e6 . "./code.cg2js/cg-calendar.fasl")
(#xdcf7ca9771c6d921f23038349c186e01 . "./code.cg2js/cg-check-box.fasl")
(#x2d48a54f5c621f506f6a54ff0fe2ac06 . "./code.cg2js/cg-acache.fasl")
(#xce562789f6aac61d6d6219cb253deb8f . "./code.cg2js/cg-wrap-string.fasl")
(#xe42dc3338b8951caf6f1583835dfddae . "./code.cg2js/cg-bitmap-stream.fasl")
(#x08ed270aac6c0c9cca9a07c34009da1e . "./code.cg2js/cg-font-dialog.fasl")
(#x4b2371c9fbbdcdf57aef54fce93f5bf1 . "./code.cg2js/cg-outline.fasl")
(#xcd53b24ea47ba80d74711307a45e1214 . "./code.cg2js/cg-shortcut-menu.fasl")
(#x042e3f43e643f6523ccb38b24ebd66c0 . "./code.cg2js/cg-status-bar.fasl")
(#xab90279d13c49da79bfb6c3e1bba93de . "./code.cg2js/cg-multi-picture-button.fasl")
(#x306faf4fcce04d8e3817ffeb7707ca77 . "./code.cg2js/cg-hotspot.fasl")
(#xd86b470167ea7a2690e30489992d472b . "./code.cg2js/cg-scaling-stream.fasl")
(#x17552fd1242825d5998df704b741bb72 . "./code.cg2js/cg-icon-pixmap.fasl")
(#x381d3fe2408ff3e9d7eceb87daa6da6d . "./code.cg2js/cg-file-dialog.fasl")
(#x1c1c9a8d560759439898fcc5db2aa77b . "./code.cg2js/cg-color-dialog.fasl")
(#x3e9efacd168faf3e5796b6e7d95a6bf4 . "./code.cg2js/cg-get-pixmap.fasl")
(#x9a2046ec7e49905a7fb35b4e153da374 . "./code.cg2js/cg-object-editor.fasl")
(#xdd0f1f0898d764806e40b22c6d5678f0 . "./code.cg2js/cg-outline-drag-and-drop.fasl")
(#x859e61977250b1750ba3e1e4a5b0c2b9 . "./code.cg2js/cg-pixmap-file-io.fasl")
(#xf484248d891f8b6b7c661f62704862a7 . "./code.cg2js/cg-pixmap.fasl")
(#xb6273b49994d26cb294827ba265466c5 . "./code.cg2js/cg-intersect.fasl")
(#xe27021807b567fa4367dc97624636c7a . "./code.cg2js/cg-scrolling-static-text.fasl")
(#xcb562266260f7feee093c8a2a1246376 . "./code.cg2js/cg-icon.fasl")
(#x5deb1452a8aada141eb1bc2eb9dbfefd . "./code.cg2js/cg-yes-no-string.fasl")
(#x96416b03e1a4de1bf58af134fed1d36b . "./code.cg2js/cg-text-or-combo.fasl")
(#x9c11ee6e3c0f625ed40b041fc1148ef9 . "./code.cg2js/cg.fasl")
(#x6f06a12b0df64eb863524c959401a309 . "./code.cg2js/cg-dialog-item.fasl")
(#xd506969a32e34d7e86026855bf11cc11 . "./code.cg2js/cg-drawable-clipboard.fasl")
(#xaab3f0de3df2d216b17e16cf4cd81c6b . "./code.cg2js/cg-caret.fasl")
(#xc35dd69bba04baa00a98598e919a567c . "./code.cg2js/cg-editable-text.fasl")
(#xf38d3a100f4bfffbb6c2b21be1d3f2e5 . "./code.cg2js/cg-choose-printer.fasl")
(#x1ba0cb24636627a128b17205c8c906c4 . "./code.cg2js/cg-yes-no-list.fasl")
(#x20de86e9a594c50e896c06dfe2e39549 . "./code.cg2js/cg-intersect-posbox.fasl")
(#x09d2089751b9719314118a7197916c84 . "./code.cg2js/cg-tooltip.fasl")
(#xbda4cbf381940b566ff9fe82560774d9 . "./code.cg2js/cg-class-slot-grid.fasl")
(#x94fed5c0e35b8ac8a209bfb4d46c95cf . "./code.cg2js/cg-combo-box.fasl")
(#x170d1df452d73b2e58dc968a7172334f . "./code.cg2js/cg-curve.fasl")
(#x49e121e0f86936e163d34545c17b8f7b . "./code.cg2js/cg-web-browser.fasl")
(#xa3d4d3808c51e71f8e637d3f79dd5ccb . "./code.cg2js/cg-printing.fasl")
(#x669c1c9350ab84702a322af5b13e653d . "./code.cg2js/cg-lisp-widget.fasl")
(#x3fa1defdcd6a4b470f5e64366d43e93f . "./code.cg2js/cg-sample-file-menu.fasl")
(#xbff3ac66ccc6b246dcc4a1bb0d9f6e05 . "./code.cg2js/cg-palette.fasl")
(#xac3068cd742f635c95388df3b2afa4bf . "./code.cg2js/cg-os-widget.fasl")
(#xc685a821a6e3de76e5650dbd665c7577 . "./code.cg2js/cg-directory-dialog-os.fasl")
(#x358d1098a889e1931f082f6f870f45f3 . "./code.cg2js/cg-pixmap-printing.fasl")
(#x96e11f91e7d468d5fb19058a17a30125 . "./code.cg2js/cg-button.fasl")
(#x39dcddb1cce83fa3156561b4a9226eb8 . "./code.cg2js/cg-edit-in-place.fasl")
(#xc3bf4c44f2961679f230cb79bceb67c3 . "./code.cg2js/cg-text-edit-pane-file-io.fasl")
(#x5913f5afae75b224d76eab0a9ed56923 . "./code.cg2js/cg-comtab.fasl")
(#x4262b575bd163d59fa457ccb63458bed . "./code.cg2js/cg-string-dialog.fasl")
(#x114f7fbf7e7945f647ce65607f871e06 . "./code.cg2js/cg-header-control.fasl")
(#xed64a21026a59b16800b1b4a161770b9 . "./code.cg2js/cg-clipboard-pixmap.fasl")
(#xc6f74a17be6b488f6883aaf165380964 . "./code.cg2js/cg-up-down-control.fasl")
(#x870dd463d074017f4b46f02e754db477 . "./code.cg2js/cg-layout.fasl")
(#x775b5f6afaae84303c54694902aeecf9 . "./code.cg2js/cg-chart-widget.fasl")
(#x45b9907bf2f2a6fddd8bb3def25ea981 . "./code.cg2js/cg-lisp-text.fasl")
(#x7e80423f64775497b119e6ea07450c6f . "./code.cg2js/cg-lisp-edit-pane.fasl")
(#x957b9726c887faed67eaf40c3493d53e . "./code.cg2js/cg-progress-indicator.fasl")
(#xf26d3e2036bd097e0b766345f5eb35ab . "./code.idejs/ide-inspector.fasl")
(#x537a173a5e262ed176da78721267b229 . "./code.idejs/ide-grapher.fasl")
(#x7d254aac8ee2e7f88dadfae41746b97a . "./code.idejs/ide-base.fasl")
(#x592158a700a63afc89456e9a176eb781 . "./code.idejs/ide-keyshort.fasl")
(#xba12fd85a4af18ed5ed2171c8024ab94 . "./code.idejs/ide-dev.fasl")
(#x6f07391e18c56467d170211710074b9e . "./code.idejs/ide-class-browser.fasl")
(#x8bdbc327a32c82b3ab1a5f60b06a3784 . "./code.idejs/ide-package-list.fasl")
(#x0cdd6d4bea6e313366b99ae3cf6e47e8 . "./code.idejs/ide-intro.fasl")
(#x05068ba2107afed8ab47093b34df0d12 . "./code.idejs/ide-class-grapher.fasl")
(#x3f905b185480d80b330caef2f4fbce62 . "./code.idejs/ide-window-list.fasl")
(#x1a6632d78f446c5c4563fc2a022af6e6 . "./code.idejs/ide-find-definitions.fasl")
(#xcda9f188fa9cd39db57e76634fda9763 . "./code.idejs/ide-trace.fasl")
(#x6135d739f2f2115816283abd88621d66 . "./code.idejs/ide-editor.fasl")
(#xa526c88586b39ccd8916227010c8a842 . "./code.idejs/ide-patch-dialog.fasl")
(#xa738703c57f4fce887d38ef0f61732ca . "./code.idejs/ide-project.fasl")
(#xb7024935bc41eb9a7ed15f01b1d5303e . "./code.idejs/ide.fasl")
(#xe896a2714328c4839417883115981b15 . "./code.idejs/ide-apropos.fasl")
(#xffaf6b90a230c00012ceb9d9380d575a . "./code.idejs/ide-acache.fasl")
(#xa3816c5903f60a555f5d10aef4af3979 . "./code.idejs/ide-clipboard.fasl")
(#x14981395336c6b80c5b46888b2169289 . "./code.idejs/ide-process-browser.fasl")
(#x6cf60b36f1eff38a3eadd600a5df9faf . "./code.idejs/ide-find-in-files.fasl")
(#xfe9573f820ab488e3265acc98555d168 . "./code.idejs/ide-form.fasl")
(#x20341be8c324a62b7a6fa36afd7f62e6 . "./code.idejs/ide-navigator.fasl")
(#x785b009207cd19fe54c93fc535e484ec . "./cgjs.html")
(#x34779e5d4bbb71d45bf63f88fac54100 . "./ssl-not-found.html")
(#x90c6d2a65504d45266ad9fa3a49be2f9 . "./browser-not-supported.html"))
 )

(let ((files-ok
       (dolist (checksum *checksums* t)
	 (when (/= (car checksum)
		   (md5-file
		    (merge-pathnames (cdr checksum) *load-pathname*)))
	   (return nil)))))
  (cond
   ((not files-ok)
    (error "The extracted files failed the checksum test.  ~
Please download and/or extract again."))
   (t
    (format t "Checksums on files are OK...~%"))))

;; We have validated all the files are there...

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; check for needed patches

(eval-when (compile eval load) (require :build))

(defvar *required-patches* '(#+macosx ("build" . 3)))

(dolist (required-patch *required-patches*)
  (let ((existing-patch (assoc (car required-patch)
			    (cdr (assoc :system sys::*patches*))
			    :test #'string=)))
    (when (null existing-patch)
      (error "Could not find patch info for the ~s module."
	     (car required-patch)))
    (when (not (>= (second existing-patch) (cdr required-patch)))
      ;; The build patch number is NOT >= to the one we require
      (error "~
Expected version ~d of ~s but have version ~d.  Please update ~
your lisp with update.~a and then do the CG/JS installation again."
	     (cdr required-patch) (car required-patch)
	     (second existing-patch)
	     #+mswindows "exe"
	     #-mswindows "sh"))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; copy files

(defun copy (source dest)
  (cond
   ((file-directory-p source)
    (format t "Copy directory ~a~%" (enough-namestring source))
    (copy-directory source dest :quiet t))
   (t
    (format t "Copy file ~a~%" (enough-namestring source))
    (sys:copy-file source dest :verbose nil))))

(defvar *sys* (translate-logical-pathname (pathname "sys:")))

(cond
 ((excl::pathname-equalp (truename *sys*)
			 (path-pathname *load-truename*))
  (format t "Files are already in place, no copying requiring.~%"))
 (t ;; copy files, replacing them with new ones if they already exist
  (let (dest)
    (dolist (from *extra-files*)
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      (when (probe-file dest) (delete-file dest))
      (copy from dest))
    
    (dolist (from (list *cg2js-directory* *idejs-directory*))
      (setq dest (merge-pathnames (file-namestring from) *sys*))
      ;; now make it a directory pathname
      (setq dest (pathname-as-directory dest))

      (when (probe-file dest) (delete-directory-and-files dest))
      (copy (pathname-as-directory from) dest)))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; update develenv.cl to load CG/JS before the original version of CG

(defvar *setup-cgjs-loading* "sys:setup-cgjs-loading.cl")

;; Because users might use an ANSI lisp to install, have to make sure
;; the resulting file can be loaded into an ANSI and Modern lisp.
(with-open-file (s *setup-cgjs-loading* :direction :output
		 :if-exists :supersede)
  (format s "~
;; This file was created by install-cgjs.cl.
;; Please do not edit it.

")
  (dolist (dir (list "code.cg2js/" "code.idejs/"))
    (format s "
(let ((path (make-pathname
	     :type \"fasl\"
	     :directory (pathname-directory
			 (merge-pathnames
			  ~s
			  (translate-logical-pathname \"sys:\"))))))
  (format t \";; *require-search-list*: adding: ~~s~%\" path)
  (push path sys:*require-search-list*))

"
	    dir)))

(defvar *develenv.cl* (pathname "sys:develenv.cl"))

(when (not (probe-file *develenv.cl*))
  (setf (file-contents *develenv.cl*)
    ";; This file is loaded before images are rebuilt with update.sh/update.exe

"))

(let ((contents (file-contents *develenv.cl*)))
  (cond
   ((match-re
     (format nil "^#\\|\\|# \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl already correctly loads CG/JS.~%"))
   ((match-re
     (format nil "^#\\+ignore \\(load \"sys:~a"
	     (file-namestring *setup-cgjs-loading*))
     contents
     :multiple-lines t)
    (format t "develenv.cl: CG/JS disabled; will enable.~%")
    (setf (file-contents *develenv.cl*)
      (replace-re  contents
		  (format nil "^#\\+ignore *\\(load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  (format nil "#||# (load \"sys:~a"
			  (file-namestring *setup-cgjs-loading*))
		  :multiple-lines t
		  :count 1)))
   (t
    (format t "develenv.cl: adding CG/JS initialization.~%")
    (setf (file-contents *develenv.cl*)
      (concatenate 'simple-string
	contents
	(format nil "~

;; To disable the use of CG/JS and go back to the originally released
;; (and possibly patched) version of CG/IDE, change the #||# to
;; #+ignore on the following line and the rebuild images by running
;; update.sh or update.exe, depending on your platform:
#||# (load \"sys:~a\")~%~%"
		(file-namestring *setup-cgjs-loading*)))))))

;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;; success!

(let ((msg (format nil "~
CG/JS installation successful!

NOTE: to use CG/JS you must rebuild images by running 'update.~a'
      in the Allegro CL directory (~a)."
		   #+mswindows "exe"
		   #-mswindows "sh"
		   *sys*)))
  #-mswindows (format t "~&~%~a~%" msg)
  #+mswindows (excl::internal-message-box msg "CG/JS Installation" :type :ok))
